@@ -38,7 +38,7 @@ from hypothesis.errors import (
     InvalidState,
 )
 from hypothesis.internal.compat import integer_types, quiet_raise, string_types
-from hypothesis.internal.reflection import get_pretty_function_description, proxies
+from hypothesis.internal.reflection import get_pretty_function_description
 from hypothesis.internal.validation import check_type, try_convert
 from hypothesis.utils.conventions import UniqueIdentifier, not_set
 from hypothesis.utils.dynamicvariables import DynamicVariable
@@ -95,7 +95,7 @@ class settingsProperty(object):
         return "\n\n".join(
             [
                 description,
-                "default value: %s" % (default,),
+                "default value: ``%s``" % (default,),
                 (deprecation_message or "").strip(),
             ]
         ).strip()
@@ -149,6 +149,7 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
     _WHITELISTED_REAL_PROPERTIES = ["_construction_complete", "storage"]
     __definitions_are_locked = False
     _profiles = {}  # type: dict
+    __module__ = "hypothesis"
 
     def __getattr__(self, name):
         if name in all_settings:
@@ -158,6 +159,10 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
 
     def __init__(self, parent=None, **kwargs):
         # type: (settings, **Any) -> None
+        if parent is not None and not isinstance(parent, settings):
+            raise InvalidArgument(
+                "Invalid argument: parent=%r is not a settings instance" % (parent,)
+            )
         if kwargs.get("derandomize"):
             if kwargs.get("database") is not None:
                 raise InvalidArgument(
@@ -196,23 +201,12 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
     def __call__(self, test):
         """Make the settings object (self) an attribute of the test.
 
-        The settings are later discovered by looking them up on the test
-        itself.
-
-        Also, we want to issue a deprecation warning for settings used alone
-        (without @given) so, note the deprecation in the new test, but also
-        attach the version without the warning as an attribute, so that @given
-        can unwrap it (since if @given is used, that means we don't want the
-        deprecation warning).
-
-        When it's time to turn the warning into an error, we'll raise an
-        exception instead of calling note_deprecation (and can delete
-        "test(*args, **kwargs)").
+        The settings are later discovered by looking them up on the test itself.
         """
         if not callable(test):
             raise InvalidArgument(
                 "settings objects can be called as a decorator with @given, "
-                "but test=%r" % (test,)
+                "but decorated test=%r is not callable." % (test,)
             )
         if inspect.isclass(test):
             from hypothesis.stateful import GenericStateMachine
@@ -234,6 +228,8 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
                     "functions, or on subclasses of GenericStateMachine."
                 )
         if hasattr(test, "_hypothesis_internal_settings_applied"):
+            # Can't use _hypothesis_internal_use_settings as an indicator that
+            # @settings was applied, because @given also assigns that attribute.
             raise InvalidArgument(
                 "%s has already been decorated with a settings object."
                 "\n    Previous:  %r\n    This:  %r"
@@ -245,29 +241,8 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
             )
 
         test._hypothesis_internal_use_settings = self
-
-        # For double-@settings check:
         test._hypothesis_internal_settings_applied = True
-
-        @proxies(test)
-        def new_test(*args, **kwargs):
-            raise InvalidArgument(
-                "Using `@settings` on a test without `@given` is completely pointless."
-            )
-
-        # @given will get the test from this attribution (rather than use the
-        # version with the deprecation warning)
-        new_test._hypothesis_internal_test_function_without_warning = test
-
-        # This means @given has been applied, so we don't need to worry about
-        # warning for @settings alone.
-        has_given_applied = getattr(test, "is_hypothesis_test", False)
-        test_to_use = test if has_given_applied else new_test
-        test_to_use._hypothesis_internal_use_settings = self
-        # Can't use _hypothesis_internal_use_settings as an indicator that
-        # @settings was applied, because @given also assigns that attribute.
-        test._hypothesis_internal_settings_applied = True
-        return test_to_use
+        return test
 
     @classmethod
     def _define_setting(
@@ -515,8 +490,8 @@ settings._define_setting(
     show_default=False,
     description="""
 An instance of hypothesis.database.ExampleDatabase that will be
-used to save examples to and load previous examples from. May be None
-in which case no storage will be used, `:memory:` for an in-memory
+used to save examples to and load previous examples from. May be ``None``
+in which case no storage will be used, ``":memory:"`` for an in-memory
 database, or any path for a directory-based example database.
 """,
     validator=_validate_database,
@@ -530,6 +505,9 @@ class Phase(IntEnum):
     generate = 2
     shrink = 3
 
+    def __repr__(self):
+        return "Phase.%s" % (self.name,)
+
 
 @unique
 class HealthCheck(Enum):
@@ -574,14 +552,7 @@ class HealthCheck(Enum):
 
     not_a_test_method = 8
     """Checks if :func:`@given <hypothesis.given>` has been applied to a
-    method of :class:`python:unittest.TestCase`."""
-
-
-@unique
-class Statistics(IntEnum):
-    never = 0
-    interesting = 1
-    always = 2
+    method defined by :class:`python:unittest.TestCase` (i.e. not a test)."""
 
 
 @unique
@@ -605,12 +576,13 @@ settings._define_setting(
 
 def _validate_phases(phases):
     if phases is None:
-        return tuple(Phase)
+        phases = tuple(Phase)
+        note_deprecation("Use phases=%r, not None." % (phases,), since="2019-08-05")
     phases = tuple(phases)
     for a in phases:
         if not isinstance(a, Phase):
             raise InvalidArgument("%r is not a valid phase" % (a,))
-    return phases
+    return tuple(p for p in list(Phase) if p in phases)
 
 
 settings._define_setting(
@@ -618,7 +590,7 @@ settings._define_setting(
     default=tuple(Phase),
     description=(
         "Control which phases should be run. "
-        + "See :ref:`the full documentation for more details <phases>`"
+        "See :ref:`the full documentation for more details <phases>`"
     ),
     validator=_validate_phases,
 )
@@ -626,7 +598,7 @@ settings._define_setting(
 settings._define_setting(
     name="stateful_step_count",
     default=50,
-    validator=lambda x: _ensure_positive_int(x, "stateful_step_count", "RELEASEDAY"),
+    validator=lambda x: _ensure_positive_int(x, "stateful_step_count", "2019-03-06"),
     description="""
 Number of steps to run a stateful program for before giving up on it breaking.
 """,
@@ -665,7 +637,7 @@ def validate_health_check_suppressions(suppressions):
 settings._define_setting(
     "suppress_health_check",
     default=(),
-    description="""A list of health checks to disable.""",
+    description="""A list of :class:`~hypothesis.HealthCheck` items to disable.""",
     validator=validate_health_check_suppressions,
 )
 
@@ -674,7 +646,8 @@ class duration(datetime.timedelta):
     """A timedelta specifically measured in milliseconds."""
 
     def __repr__(self):
-        return "timedelta(milliseconds=%r)" % (self.total_seconds() * 1000,)
+        ms = self.total_seconds() * 1000
+        return "timedelta(milliseconds=%r)" % (int(ms) if ms == int(ms) else ms,)
 
 
 def _validate_deadline(x):
@@ -734,15 +707,8 @@ class PrintSettings(Enum):
     """Never print a blob."""
 
     INFER = 1
-    """Make an educated guess as to whether it would be appropriate to print
-    the blob.
-
-    The current rules are that this will print if:
-
-    1. The output from Hypothesis appears to be unsuitable for use with
-       :func:`~hypothesis.example`, and
-    2. The output is not too long, and
-    3. Verbosity is at least normal."""
+    """This option is deprecated and will be treated as equivalent to
+    ALWAYS."""
 
     ALWAYS = 2
     """Always print a blob on failure."""
@@ -752,11 +718,8 @@ class PrintSettings(Enum):
 
 
 def _validate_print_blob(value):
-    if isinstance(value, bool):
-        if value:
-            replacement = PrintSettings.ALWAYS
-        else:
-            replacement = PrintSettings.NEVER
+    if isinstance(value, PrintSettings):
+        replacement = value != PrintSettings.NEVER
 
         note_deprecation(
             "Setting print_blob=%r is deprecated and will become an error "
@@ -766,23 +729,19 @@ def _validate_print_blob(value):
         )
         return replacement
 
-    # Values that aren't bool or PrintSettings will be turned into hard errors
-    # by the 'options' check.
+    check_type(bool, value, "print_blob")
+
     return value
 
 
 settings._define_setting(
     "print_blob",
-    default=PrintSettings.INFER,
+    default=False,
     description="""
-Determines whether to print blobs after tests that can be used to reproduce
-failures.
-
-See :ref:`the documentation on @reproduce_failure <reproduce_failure>` for
-more details of this behaviour.
+If set to True, Hypothesis will print code for failing examples that can be used with
+:func:`@reproduce_failure <hypothesis.reproduce_failure>` to reproduce the failing example.
 """,
     validator=_validate_print_blob,
-    options=tuple(PrintSettings),
 )
 
 settings.lock_further_definitions()
